
-- =====================================================================
-- MFM-Doktor
-- Repair MFM files that do not load correctly in MSEide due to invalid
-- values for properties.
-- =====================================================================

local appname = 'MFM-Doktor 0.0.2 (' .. _VERSION .. ')\n'
local help = 'Usage:\n  lua mfmdok.lua <input>.mfm <output>.mfm\n'

function set(list)
  local result = {}
  for i, v in ipairs(list) do
    result[v] = true
  end
  return result
end

local windowoptionty = set({
  'wo_buttonendmodal',
  'wo_combo',
  'wo_desktop',
  'wo_dialog',
  'wo_dnd',
  'wo_dock',
  'wo_dropdownmenu',
  'wo_embedded',
  'wo_groupleader',
  'wo_menu',
  'wo_message',
  'wo_noactivate',
  'wo_noframe',
  'wo_notaskbar',
  'wo_notification',
  'wo_overrideredirect',
  'wo_popup',
  'wo_popupmenu',
  'wo_splash',
  'wo_sysdnd',
  'wo_taskbar',
  'wo_toolbar',
  'wo_tooltip',
  'wo_utility',
  'wo_windowcentermessage'
})

local optionwidgetty = set({
  'ow_appinactivehint',
  'ow_arrowfocus',
  'ow_arrowfocusin',
  'ow_arrowfocusout',
  'ow_background',
  'ow_canclosenil',
  'ow_destroywidgets',
  'ow_disabledhint',
  'ow_focusbackonesc',
  'ow_hintoff',
  'ow_hinton',
  'ow_keyreturntaborder',
  'ow_mousefocus',
  'ow_mousetransparent',
  'ow_mousewheel',
  'ow_multiplehint',
  'ow_noautosizing',
  'ow_nochildclipsiblings',
  'ow_nochildpaintclip',
  'ow_nochildshortcut',
  'ow_nohidewidgets',
  'ow_noparentshortcut',
  'ow_noscroll',
  'ow_nosiblingshortcut',
  'ow_parenttabfocus',
  'ow_subfocus',
  'ow_tabfocus',
  'ow_timedhint',
  'ow_top',
  'ow_transientformain',
  'ow_ultratop'
})

local optionwidget1ty = set({
  'ow1_autoheight',
  'ow1_autoscale',
  'ow1_autosizeanbottom',
  'ow1_autosizeanright',
  'ow1_autowidth',
  'ow1_canclosenil',
  'ow1_clientcxmax',
  'ow1_clientcxmin',
  'ow1_clientcymax',
  'ow1_clientcymin',
  'ow1_fontglyphheight',
  'ow1_fontlineheight',
  'ow1_invisibleparentsizeextend',
  'ow1_modalcallonactivate',
  'ow1_modalcallondeactivate',
  'ow1_noassistive',
  'ow1_noautosizeanbottom',
  'ow1_noautosizeanright',
  'ow1_noautosizing',
  'ow1_nocancloseifhidden',
  'ow1_noclampinview',
  'ow1_noparentheightextend',
  'ow1_noparentwidthextend'
})

function RemoveInvalidValues(text, optionprefix, validvalues)
  local i, j = text:find(optionprefix .. '_%a+')
  while i ~= nil do
    local word = text:sub(i, j)
    if validvalues[word] then
      io.write('DEBUG Valid value: ' .. word .. '\n')
      i, j = text:find(optionprefix .. '_%a+', j + 1)
    else
      io.write('DEBUG Invalid value: ' .. word .. '\n')
      local k, l = text:find(word .. ', ', i)
      if k ~= nil then
        io.write('DEBUG Delete: [' .. string.sub(text, k, l) .. ']\n')
        text = string.sub(text, 1, k - 1) .. string.sub(text, l + 1)
      else
        k, l = text:find(', ' .. word, i - 2)
        if k ~= nil then
          io.write('DEBUG Delete: [' .. string.sub(text, k, l) .. ']\n')
          text = string.sub(text, 1, k - 1) .. string.sub(text, l + 1)
        else
          io.write('DEBUG Delete: [' .. string.sub(text, i, j) .. ']\n')
          text = string.sub(text, 1, i - 1) .. string.sub(text, j + 1)
        end
      end
      i, j = text:find(optionprefix .. '_%a+', i)
    end
  end
  return text
end

io.write(appname)

if #arg == 2 then
  local file = io.open(arg[1], 'r')
  local text = file:read('*a')
  file:close()
  
  text = RemoveInvalidValues(text, 'wo', windowoptionty)
  text = RemoveInvalidValues(text, 'ow', optionwidgetty)
  text = RemoveInvalidValues(text, 'ow1', optionwidget1ty)
  
  file = io.open(arg[2], 'w')
  file:write(text)
  file:close()
else
  io.write(help)
end
